@@ -71,7 +71,7 @@ usage() {
     echo ""
 }
 
-NVIDIA_BUG_REPORT_CHANGE='$Change: 31831145 $'
+NVIDIA_BUG_REPORT_CHANGE='$Change: 34323106 $'
 NVIDIA_BUG_REPORT_VERSION=`echo "$NVIDIA_BUG_REPORT_CHANGE" | tr -c -d "[:digit:]"`
 
 # Set the default filename so that it won't be empty in the usage message
@@ -124,27 +124,32 @@ echo_metadata() {
     fi
 }
 
+#
+# report_file() - print a report for the specified file to stdout: if it
+# exists, dumps metadata and contents.
+#
+report_file() {
+    echo "____________________________________________"
+    echo ""
+
+    if [ ! -f "$1" ]; then
+        echo "*** $1 does not exist"
+    elif [ ! -r "$1" ]; then
+        echo "*** $1 is not readable"
+    else
+        echo "*** $1"
+        echo_metadata "$1"
+        cat  "$1"
+    fi
+    echo ""
+}
 
 #
 # append() - append the contents of the specified file to the log
 #
 
 append() {
-    (
-        echo "____________________________________________"
-        echo ""
-
-        if [ ! -f "$1" ]; then
-            echo "*** $1 does not exist"
-        elif [ ! -r "$1" ]; then
-            echo "*** $1 is not readable"
-        else
-            echo "*** $1"
-            echo_metadata "$1"
-            cat  "$1"
-        fi
-        echo ""
-    ) | $GZIP_CMD >> $LOG_FILENAME
+    report_file "$1" | $GZIP_CMD >> $LOG_FILENAME
 }
 
 #
@@ -167,13 +172,13 @@ append_silent() {
 
 #
 # append_glob() - use the shell to expand a list of files, and invoke
-# append() for each of them
+# report_file for each of them; append the result to the log
 #
 
 append_glob() {
     for append_glob_iterator in `ls $1 2> /dev/null;`; do
-        append "$append_glob_iterator"
-    done
+        report_file "$append_glob_iterator"
+    done | $GZIP_CMD >> $LOG_FILENAME
 }
 
 #
@@ -217,10 +222,10 @@ append_binary_file() {
 }
 
 #
-# append_command() - append the output of the specified command to the log
+# report_command() - print the output of the specified command to stdout
 #
 
-append_command() {
+report_command() {
     if [ -n "$1" ]; then
         echo "$1"
         echo ""
@@ -359,7 +364,7 @@ print_libglvnd_library()
 #
 
 
-# check that we are root (needed for `lspci -vxxx` and potentially for
+# check that we are root (needed for `lspci -vxxxx` and potentially for
 # accessing kernel log files)
 
 if [ `id -u` -ne 0 ]; then
@@ -428,26 +433,44 @@ echo -n "Running $(basename $0)...";
     echo ""
 ) | $GZIP_CMD >> $LOG_FILENAME
 
-# List the sysfs entries for all NVIDIA device functions
-# This info is useful to debug dynamic power management issues
-#
-# NOTE: We need to query this before other things in this script,
-# because other operations may alter the power management
-# state of the GPU(s).
-for subdir in `ls /sys/bus/pci/devices/ 2> /dev/null`; do
-    vendor_id=`cat /sys/bus/pci/devices/$subdir/vendor 2> /dev/null`
-    if [ "$vendor_id" = "0x10de" ]; then
-        append "/sys/bus/pci/devices/$subdir/power/control"
-        append "/sys/bus/pci/devices/$subdir/power/runtime_status"
-        append "/sys/bus/pci/devices/$subdir/power/runtime_usage"
-    fi
-done
+if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
+    # List the sysfs entries for all NVIDIA device functions
+    # This info is useful to debug dynamic power management issues
+    #
+    # NOTE: We need to query this before other things in this script,
+    # because other operations may alter the power management
+    # state of the GPU(s).
+    for subdir in `ls /sys/bus/pci/devices/ 2> /dev/null`; do
+        vendor_id=`cat /sys/bus/pci/devices/$subdir/vendor 2> /dev/null`
+        if [ "$vendor_id" = "0x10de" ]; then
+            append "/sys/bus/pci/devices/$subdir/power/control"
+            append "/sys/bus/pci/devices/$subdir/power/runtime_status"
+            append "/sys/bus/pci/devices/$subdir/power/runtime_usage"
 
-for subdir in $proc_module_dirs; do
-    for GPU in `ls /proc/driver/nvidia/$subdir/gpus/ 2> /dev/null`; do
-        append "/proc/driver/nvidia/$subdir/gpus/$GPU/power"
+            # Get the parent port's power resources to determine
+            # if parent port can control the power. We check the parent port
+            # only for the first subfunction (VGA/3D).
+            pci_class=`cat /sys/bus/pci/devices/$subdir/class 2> /dev/null`
+            if [ "$pci_class" = "0x030000" ] || [ "$pci_class" = "0x030200" ]; then
+                path_to_device=`readlink -f /sys/bus/pci/devices/$subdir`
+                pr_d3hot="$path_to_device/../firmware_node/power_resources_D3hot"
+                (
+                    echo "____________________________________________"
+                    echo ""
+                    echo "power_resources_d3hot directory of the parent PCI/e port"
+                    echo ""
+                    echo_metadata $pr_d3hot
+                ) | $GZIP_CMD >> $LOG_FILENAME
+            fi
+        fi
     done
-done
+
+    for subdir in $proc_module_dirs; do
+        for GPU in `ls /proc/driver/nvidia/$subdir/gpus/ 2> /dev/null`; do
+            append "/proc/driver/nvidia/$subdir/gpus/$GPU/power"
+        done
+    done
+fi
 
 # append OPAL (IBM POWER system firmware) messages
 
@@ -668,35 +691,36 @@ append_file_or_dir_silent /usr/share/nvidia/nvidia-application-profiles-*-rc
     lspci=`which lspci 2> /dev/null | head -n 1`
 
     if [ $? -eq 0 -a -x "$lspci" ]; then
-        # Capture NVIDIA devices
-        echo "$lspci -d \"10de:*\" -v -xxx"
+        # Capture all devices in tree format along with vendor:device IDs
+        echo "$lspci -nntv"
         echo ""
-        $lspci -d "10de:*" -v -xxx 2> /dev/null
-        echo ""
-        echo "____________________________________________"
-        echo ""
-        # Capture PLX devices
-        echo "$lspci -d \"10b5:*\" -v -xxx"
-        echo ""
-        $lspci -d "10b5:*" -v -xxx 2> /dev/null
-        echo ""
-        echo "____________________________________________"
-        echo ""
-        echo "$lspci -t"
-        echo ""
-        $lspci -t 2> /dev/null
+        $lspci -nntv 2> /dev/null
         echo ""
         echo "____________________________________________"
         echo ""
+        # Capture class names and class ID  along with vendor:device IDs
         echo "$lspci -nn"
         echo ""
         $lspci -nn 2> /dev/null
+        echo ""
+        echo "____________________________________________"
+        echo ""
+        # Capture verbose information for all devices, along with hex
+        # dump of whole configuration space
+        echo "$lspci -nnDvvvxxxx"
+        echo ""
+        $lspci -nnDvvvxxxx 2> /dev/null
     else
         echo "Skipping lspci output (lspci not found)"
         echo ""
     fi
 ) | $GZIP_CMD >> $LOG_FILENAME
 
+# THP (transparent huge pages) information
+append /sys/kernel/mm/transparent_hugepage/enabled
+
+append /proc/sys/vm/compaction_proactiveness
+
 # NUMA information
 
 (
@@ -706,42 +730,35 @@ append_file_or_dir_silent /usr/share/nvidia/nvidia-application-profiles-*-rc
     numactl=`which numactl 2> /dev/null | head -n 1`
 
     if [ $? -eq 0 -a -x "$numactl" ]; then
-	# Get hardware NUMA configuration
-	echo "$numactl -H"
-	echo ""
-	$numactl -H
+        # Get hardware NUMA configuration
+        echo "$numactl -H"
+        echo ""
+        $numactl -H
     fi
 
-    # Get additional NUMA information
+    # Get autonuma information:
+    report_file /proc/sys/kernel/numa_balancing
+
+    # Get additional NUMA information about GPUs:
     filelist="/sys/devices/system/node/has_cpu \
-    	      /sys/devices/system/node/has_memory \
-	      /sys/devices/system/node/has_normal_memory \
-	      /sys/devices/system/node/online \
-	      /sys/devices/system/node/possible"
+              /sys/devices/system/node/has_memory \
+              /sys/devices/system/node/has_normal_memory \
+              /sys/devices/system/node/online \
+              /sys/devices/system/node/possible"
 
     # Get GPU NUMA information
     lspci=`which lspci 2> /dev/null | head -n 1`
     if [ $? -eq 0 -a -x "$lspci" ]; then
-	gpus=`$lspci -d "10de:*" -s ".0" | awk '{print $1}'`
-	for gpu in $gpus; do
-	    filelist="$filelist \
-	    	      /sys/bus/pci/devices/*$gpu/local_cpulist \
-		      /sys/bus/pci/devices/*$gpu/numa_node"
-	done
+        gpus=`$lspci -d "10de:*" -s ".0" | awk '{print $1}'`
+        for gpu in $gpus; do
+            filelist="$filelist \
+                      /sys/bus/pci/devices/*$gpu/local_cpulist \
+                      /sys/bus/pci/devices/*$gpu/numa_node"
+        done
     fi
 
     for file in $filelist; do
-	echo "____________________________________________"
-	if [ ! -f "$file" ]; then
-	    echo "*** $file does not exist"
-	elif [ ! -r "$file" ]; then
-	    echo "*** $file is not readable"
-	else
-	    echo "*** $file"
-	    echo_metadata "$file"
-	    cat "$file"
-	fi
-	echo ""
+        report_file "$file"
     done
 ) | $GZIP_CMD >> $LOG_FILENAME
 
@@ -895,79 +912,91 @@ fi
     fi
 ) | $GZIP_CMD >> $LOG_FILENAME
 
-# In case of failure, if xset returns with delay, we print the
-# message from check "$?" & if it returns error immediately before kill,
-# we directly write the error to the log file.
+if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
 
-(
-    echo "____________________________________________"
-    echo ""
-    echo "xset -q:"
-    echo ""
+    # In case of failure, if xset returns with delay, we print the
+    # message from check "$?" & if it returns error immediately before kill,
+    # we directly write the error to the log file.
 
-    xset -q 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1
-
-    if [ $? -eq 0 ]; then
-        # The xset process is still there.
-        echo "xset could not connect to an X server"
-    fi
-) | $GZIP_CMD >> $LOG_FILENAME
-
-# In case of failure, if nvidia-settings returns with delay, we print the
-# message from check "$?" & if it returns error immediately before kill,
-# we directly write the error to the log file.
-
-(
-    echo "____________________________________________"
-    echo ""
-    echo "nvidia-settings -q all:"
-    echo ""
-
-    DISPLAY= nvidia-settings -c "$DPY" -q all 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1
-
-    if [ $? -eq 0 ]; then
-        # The nvidia-settings process is still there.
-        echo "nvidia-settings could not connect to an X server"
-    fi
-) | $GZIP_CMD >> $LOG_FILENAME
-
-# In case of failure, if xrandr returns with delay, we print the
-# message from check "$?" & if it returns error immediately before kill,
-# we directly write the error to the log file.
-
-(
-    if [ -x "`which xrandr 2>/dev/null`" ] ; then
-         echo "____________________________________________"
-         echo ""
-         echo "xrandr --verbose:"
-         echo ""
-
-         xrandr -display $DPY --verbose 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1
-         if [ $? -eq 0 ]; then
-             # The xrandr process is still there.
-             echo "xrandr could not connect to an X server"
-         fi
-    else
-        echo "Skipping xrandr output (xrandr not found)"
-    fi
-) | $GZIP_CMD >> $LOG_FILENAME
-
-(
-    if [ -x "`which xprop 2>/dev/null`" ] ; then
+    (
         echo "____________________________________________"
         echo ""
-        echo "Running window manager properties:"
+        echo "xset -q:"
         echo ""
 
-        TMP=`xprop -root _NET_SUPPORTING_WM_CHECK 2>/dev/null & sleep 1 ; kill -9 $! > /dev/null 2>&1`
-        WINDOW=`echo $TMP | grep -o '0x[0-9a-z]\+'`
-        if [ "$WINDOW" ]; then
-            xprop -id "$WINDOW" 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1
-        else
-            echo "Unable to detect window manager properties"
+        xset -q 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1
+
+        if [ $? -eq 0 ]; then
+            # The xset process is still there.
+            echo "xset could not connect to an X server"
         fi
-    fi
-) | $GZIP_CMD >> $LOG_FILENAME
+    ) | $GZIP_CMD >> $LOG_FILENAME
+
+    # In case of failure, if nvidia-settings returns with delay, we print the
+    # message from check "$?" & if it returns error immediately before kill,
+    # we directly write the error to the log file.
+
+    (
+        echo "____________________________________________"
+        echo ""
+        echo "nvidia-settings -q all:"
+        echo ""
+
+        DISPLAY= nvidia-settings -c "$DPY" -q all 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1
+
+        if [ $? -eq 0 ]; then
+            # The nvidia-settings process is still there.
+            echo "nvidia-settings could not connect to an X server"
+        fi
+    ) | $GZIP_CMD >> $LOG_FILENAME
+
+    # In case of failure, if xrandr returns with delay, we print the
+    # message from check "$?" & if it returns error immediately before kill,
+    # we directly write the error to the log file.
+
+    (
+        if [ -x "`which xrandr 2>/dev/null`" ] ; then
+             echo "____________________________________________"
+             echo ""
+             echo "xrandr --verbose:"
+             echo ""
+
+             xrandr -display $DPY --verbose 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1
+             if [ $? -eq 0 ]; then
+                 # The xrandr process is still there.
+                 echo "xrandr could not connect to an X server"
+             fi
+             echo "____________________________________________"
+             echo ""
+             echo "xrandr --listproviders:"
+             echo ""
+             xrandr -display $DPY --listproviders 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1
+             if [ $? -eq 0 ]; then
+                 # The xrandr process is still there.
+                 echo "xrandr could not connect to an X server"
+            fi
+        else
+            echo "Skipping xrandr output (xrandr not found)"
+        fi
+    ) | $GZIP_CMD >> $LOG_FILENAME
+
+    (
+        if [ -x "`which xprop 2>/dev/null`" ] ; then
+            echo "____________________________________________"
+            echo ""
+            echo "Running window manager properties:"
+            echo ""
+
+            TMP=`xprop -root _NET_SUPPORTING_WM_CHECK 2>/dev/null & sleep 1 ; kill -9 $! > /dev/null 2>&1`
+            WINDOW=`echo $TMP | grep -o '0x[0-9a-z]\+'`
+            if [ "$WINDOW" ]; then
+                xprop -id "$WINDOW" 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1
+            else
+                echo "Unable to detect window manager properties"
+            fi
+        fi
+    ) | $GZIP_CMD >> $LOG_FILENAME
+fi
 
 sync > /dev/null 2>&1
 sync > /dev/null 2>&1
@@ -986,15 +1015,22 @@ append "/proc/mtrr"
 
 for subdir in $proc_module_dirs; do
     append "/proc/driver/nvidia/$subdir/version"
-    for GPU in `ls /proc/driver/nvidia/$subdir/gpus/ 2> /dev/null`; do
-        append "/proc/driver/nvidia/$subdir/gpus/$GPU/information"
-        append "/proc/driver/nvidia/$subdir/gpus/$GPU/registry"
-    done
-    append_glob "/proc/driver/nvidia/$subdir/warnings/*"
     append "/proc/driver/nvidia/$subdir/params"
     append "/proc/driver/nvidia/$subdir/registry"
 done
 
+if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
+    for subdir in $proc_module_dirs; do
+        for GPU in `ls /proc/driver/nvidia/$subdir/gpus/ 2> /dev/null`; do
+            append "/proc/driver/nvidia/$subdir/gpus/$GPU/information"
+    #VGX_SPECIFIC_CODE_START
+            append "/proc/driver/nvidia/$subdir/gpus/$GPU/unbindLock"
+    #VGX_SPECIFIC_CODE_END
+        done
+        append_glob "/proc/driver/nvidia/$subdir/warnings/*"
+    done
+fi
+
 append_glob "/proc/acpi/video/*/*/info"
 
 append "/proc/asound/cards"
@@ -1050,6 +1086,62 @@ done
     echo ""
 ) | $GZIP_CMD >> $LOG_FILENAME
 
+# nvidia-debugdump
+(
+    echo "____________________________________________"
+    echo ""
+
+    nvidia_debugdump=`which nvidia-debugdump 2> /dev/null | head -n 1`
+
+    if [ $? -eq 0 -a -x "$nvidia_debugdump" ]; then
+
+        if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
+            NVDD_ARGS="-D"
+        else
+            NVDD_ARGS="--ioctl --nvlogonly"
+        fi
+
+        base64=`which base64 2> /dev/null | head -n 1`
+
+        if [ $? -eq 0 -a -x "$base64" ]; then
+            # make sure what we can write to the temp file
+
+            NVDD_TEMP_FILENAME="nvidia-debugdump-temp$$.log"
+
+            touch $NVDD_TEMP_FILENAME 2> /dev/null
+
+            if [ $? -ne 0 ]; then
+                echo "Skipping nvidia-debugdump output (can't create temp file $NVDD_TEMP_FILENAME)"
+                echo ""
+                # don't fail here, continue
+            else
+                for instance in $proc_module_dirs; do
+                    if [ $instance != '.' ]; then
+                        export __RM_MODULE_INSTANCE=$instance
+                        echo "NVIDIA Kernel module instance $instance"
+                    fi
+
+                    echo "$nvidia_debugdump ${NVDD_ARGS}"
+                    echo ""
+                    $nvidia_debugdump ${NVDD_ARGS} -f $NVDD_TEMP_FILENAME 2> /dev/null
+                    $base64 $NVDD_TEMP_FILENAME 2> /dev/null
+                    echo ""
+
+                    # remove the temporary file when complete
+                    rm $NVDD_TEMP_FILENAME 2> /dev/null
+                    unset __RM_MODULE_INSTANCE
+                done
+            fi
+        else
+            echo "Skipping nvidia-debugdump output (base64 not found)"
+            echo ""
+        fi
+    else
+        echo "Skipping nvidia-debugdump output (nvidia-debugdump not found)"
+        echo ""
+    fi
+) | $GZIP_CMD >> $LOG_FILENAME
+
 # disable these when safemode is requested
 if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
 
@@ -1092,8 +1184,11 @@ if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
                     export __RM_MODULE_INSTANCE=$instance
                     echo "NVIDIA Kernel module instance $instance"
                 fi
-                append_command "$nvidia_smi --query"
-                append_command "$nvidia_smi --query --unit"
+                report_command "$nvidia_smi --query"
+                report_command "$nvidia_smi --query --unit"
+                report_command "$nvidia_smi nvlink --errorcounters"
+                report_command "$nvidia_smi nvlink --remotelinkinfo"
+                report_command "$nvidia_smi nvlink --status"
                 unset __RM_MODULE_INSTANCE
             done
         else
@@ -1107,61 +1202,9 @@ if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
         rm -f $NVML_LOG_FILE
         unset __NVML_DBG_FILE __NVML_DBG_APPEND __NVML_DBG_LVL
     fi
-
-    # nvidia-debugdump
-
-    (
-        echo "____________________________________________"
-        echo ""
-
-        nvidia_debugdump=`which nvidia-debugdump 2> /dev/null | head -n 1`
-
-        if [ $? -eq 0 -a -x "$nvidia_debugdump" ]; then
-        
-            base64=`which base64 2> /dev/null | head -n 1`
-            
-            if [ $? -eq 0 -a -x "$base64" ]; then
-                # make sure what we can write to the temp file
-                
-                NVDD_TEMP_FILENAME="nvidia-debugdump-temp$$.log"
-                
-                touch $NVDD_TEMP_FILENAME 2> /dev/null
-
-                if [ $? -ne 0 ]; then
-                    echo "Skipping nvidia-debugdump output (can't create temp file $NVDD_TEMP_FILENAME)"
-                    echo ""
-                    # don't fail here, continue
-                else
-                    for instance in $proc_module_dirs; do
-                        if [ $instance != '.' ]; then
-                            export __RM_MODULE_INSTANCE=$instance
-                            echo "NVIDIA Kernel module instance $instance"
-                        fi
-
-                        echo "$nvidia_debugdump -D"
-                        echo ""
-                        $nvidia_debugdump -D -f $NVDD_TEMP_FILENAME 2> /dev/null
-                        $base64 $NVDD_TEMP_FILENAME 2> /dev/null
-                        echo ""
-
-                        # remove the temporary file when complete
-                        rm $NVDD_TEMP_FILENAME 2> /dev/null
-                        unset __RM_MODULE_INSTANCE
-                    done
-                fi
-            else
-                echo "Skipping nvidia-debugdump output (base64 not found)"
-                echo ""
-            fi
-        else
-            echo "Skipping nvidia-debugdump output (nvidia-debugdump not found)"
-            echo ""
-        fi
-    ) | $GZIP_CMD >> $LOG_FILENAME
-
 else
     (
-        echo "Skipping nvidia-smi, nvidia-debugdump, and vulkaninfo due to --safe-mode argument."
+        echo "Skipping nvidia-smi, vulkaninfo etc. due to --safe-mode argument."
         echo ""
     ) | $GZIP_CMD >> $LOG_FILENAME
 fi
@@ -1181,6 +1224,23 @@ if [ $? -eq 0 -a -x "$systemctl" ]; then
     ) 2> /dev/null | $GZIP_CMD >> $LOG_FILENAME
 fi
 
+#copy IMEX daemon (for multi-node compute systems) default log file
+append_silent "/var/log/nvidia-imex.log"
+append_silent "/var/log/nvidia-imex-verbose.log"
+append_silent "/var/log/nvidia-imex-stats.log"
+# get nvidia-imex service status information
+if [ $? -eq 0 -a -x "$systemctl" ]; then
+    cmd="$systemctl status nvidia-imex.service"
+    (
+        echo "____________________________________________"
+        echo ""
+        echo "$cmd"
+        $cmd
+        echo ""
+    ) 2> /dev/null | $GZIP_CMD >> $LOG_FILENAME
+fi
+
+
 # Print information about the libglvnd libraries
 (
     if [ -e $GLVND_HELPER_BASE_PATH/glvnd_check ] ; then
